<?php 
/*
  A quick and dirty 404 handler for a missing tiles when acessing a MapGuide 
  tiled map served directly by IIS.

  Example URL:
  http://localhost:8008/sheboyganTiles/S13/Base Layer Group/R4/C4/11_7.png

  Note:
  1. This script does not handle extra parameters, i.e. the "querystring"
     parameter added under http://trac.openlayers.org/ticket/1622.
  2. You must hard-code the tile map repository definition and access
     credentials inside this file.

  Configuration:
  1. In IIS select the virtual folder containing the tile cache, right-click and
     select Properties.
  2. In the Properties dialog, select the Custom Errors tab.
  3. Choose 404 from the list and click Edit.
  4. In the Edit Custom Error Properties dialog choose URL for Message Type and 
     enter a relative path to this PHP file, e.g. "/mapguide/blah/getTile.php".
  5. Save this file in the relevant location, e.g. 
     "C:\Program Files\MapGuideOpenSource2.0\WebServerExtensions\www\blah\getTile.php".
*/

// ----------------   Setup ----------------------
// Include the MapGuide constants located at 
// C:/Program Files/MapGuideOpenSource2.0/WebServerExtensions/www/mapadmin/constants.php.
include "../../mapadmin/constants.php";

// Location of the MapGuide web extensions, e.g.
// C:/Program Files/MapGuideOpenSource2.0/WebServerExtensions/www/.
$extensionDir = "C:/Program Files/MapGuideOpenSource2.0/WebServerExtensions/www/";

// Tile Map that you want to serve.
$tileMap = "Library://Samples/Sheboygan/MapsTiled/Sheboygan.MapDefinition";

// Credentials to use.
$tileUser = 'Anonymous';
$tilePassword = '';
// ----------------   End Setup ------------------

if (!isset($_SERVER['REQUEST_URI'])){
  header("HTTP/1.x 500 expected SERVER REQUEST_URI is missing");
  print "<h1>Exception</h1><p>";
  print "<strong>Error:</strong>The expected SERVER REQUEST_URI is missing<br/>";
  print "<strong>Details:</strong> This is script is designed to be called via an IIS 404 redirect.<br/>";
  exit;
}

// Anticipated example REQUEST_URI:
// "/httpmaptest/tiles/getTile.php?404;http://localhost:80/httpmaptest/tiles/S5/Base Layer Group/R30/C0/0_6.png"
$ar_path = split("/", $_SERVER['REQUEST_URI']); 

// Pick off what we need from the right-hand side, working left.
$pathLength = sizeof($ar_path);
$imgFile = $ar_path[$pathLength - 1];
$groupCol = substr($ar_path[$pathLength - 2], 1);
$groupRow = substr($ar_path[$pathLength - 3], 1);
$baseMapLayerGroupName = $ar_path[$pathLength - 4];
$scaleIndex  = substr($ar_path[$pathLength - 5], 1);

$ar_file=split("[_.]", $imgFile);
$fileRow = $ar_file[0];
$fileCol = $ar_file[1];

$tileRow = $groupRow + $fileRow;
$tileCol = $groupCol + $fileCol;

try 
{
  MgInitializeWebTier($extensionDir . "webconfig.ini");

  // Connect to the site as the chosen user.
  $user = new MgUserInformation($tileUser, $tilePassword);
  $siteConnection = new MgSiteConnection();
  $siteConnection->Open($user);
   
  // Set up the tiling service.
  $tileService = $siteConnection->CreateService(MgServiceType::TileService);

  // Fetch the image.
  $tileResource = new MgResourceIdentifier($tileMap);
  $byteReader = $tileService->GetTile($tileResource, urldecode($baseMapLayerGroupName), $tileCol, $tileRow, $scaleIndex);

  // Output the image contents.
  header("Content-Type: image/png");
  $buffer = '';
  while ($byteReader->Read($buffer, 50000) != 0)
  {
    echo $buffer;
  }
}
catch (MgException $ex) 
{
  header("HTTP/1.0 Status: 500 " . $ex->GetMessage());
}

exit;
?>